'use strict';

const electron = require('electron');
// 控制应用生命周期的模块
const {app} = electron;
// 创建本地浏览器窗口的模块
const {BrowserWindow} = electron;
const {ipcMain} = electron;

// 指向窗口对象的一个全局引用，如果没有这个引用，那么当该javascript对象被垃圾回收的
// 时候该窗口将会自动关闭
let win;

/* ================================================================================
addon related functions
================================================================================ */
var addon = require('./lib/BackendCaller');

function Log(msg){
	msg = '[JS main]' + msg;
  console.log(msg);
}
function listenStopped ( ) {
	Log("Listen Stopped" );
}

function eventReceived ( s ) {
	//Log( s);

	//send to each sub window.
	win.webContents.send('renderCallDone',s);
}

function StartListen(){
  var libDir = __dirname + "/lib";
	Log("Start listen, directory path : " + libDir);
		
	ipcMain.on('renderCallStart', (event, name/*string */, args/*any*/, callback/*string*/) => {
		var result = addon.CallAsyncFunction(name, JSON.stringify(args), callback);
	Log("Call async function: " + name )
	});
	
	ipcMain.on('ismaximized',()=>{
	    win.webContents.send('ismaximized',win.isMaximized());
	});
	ipcMain.on('shutdown',()=>{
	    app.quit();
	});
    ipcMain.on('minimum', () => {
        win.minimize();
    });
    ipcMain.on('normal', () => {
        win.unmaximize();
    });
    ipcMain.on('maximum', () => {
        win.maximize();
    });
	addon.StartListenToResult( 10 , libDir ,eventReceived, listenStopped );
}

function StopListen(){
	var bStopListen = addon.StopListenToResult();
	Log("Stop listen : " + bStopListen);
}

/* ================================================================================
electron window related functions
================================================================================ */


function createWindow() {
  // 创建一个新的浏览器窗口, main window
  win = new BrowserWindow({width: 1024, height: 768,frame:false});

  // 并且装载应用的index.html页面
  win.loadURL('file://' + __dirname + '/app/index.html');

  // 当窗口关闭时调用的方法
  win.on('closed', () => {
    // 解除窗口对象的引用，通常而言如果应用支持多个窗口的话，你会在一个数组里
    // 存放窗口对象，在窗口关闭的时候应当删除相应的元素。
    win = null;
  });
}

// 当Electron完成初始化并且已经创建了浏览器窗口，则该方法将会被调用。
// 有些API只能在该事件发生后才能被使用。
app.on('ready', ()=>{
  if (process.platform === 'win32') {
    app.setAppUserModelId('com.BazBull.DevilfishCommander');
  }

	StartListen();
	
	createWindow();
});

// 当所有的窗口被关闭后退出应用
app.on('window-all-closed', () => {

  // 对于OS X系统，应用和相应的菜单栏会一直激活直到用户通过Cmd + Q显式退出
  if (process.platform !== 'darwin') {
    app.quit();
  }
});

app.on('activate', () => {
  // 对于OS X系统，当dock图标被点击后会重新创建一个app窗口，并且不会有其他
  // 窗口打开
  if (win === null) {
    createWindow();
  }
});


// 应用退出时关闭后端监听
app.on('quit', StopListen);
